import grok
from zope.component.interfaces import IFactory
from waeup.interfaces import IFaculty
from waeup.viewlets import MainArea, LeftSidebar, Index, FormWrapMixin, Manage

class Faculty(grok.Container):
    grok.implements(IFaculty)

    def __init__(self,
                 title=u'Unnamed Faculty',
                 title_prefix=u'faculty',
                 code=None, **kw):
        super(Faculty, self).__init__(**kw)
        self.title = title
        self.title_prefix = title_prefix
        self.code = code

class FacultyFactory(grok.GlobalUtility):
    """A factory for faculty containers.
    """
    grok.implements(IFactory)
    grok.name(u'waeup.Faculty')
    title = u"Create a new faculty.",
    description = u"This factory instantiates new faculty instances."

    def __call__(self):
        return Faculty()

    def getInterfaces(self):
        return implementedBy(Faculty)

        
class DisplayForm(grok.DisplayForm):
    grok.context(IFaculty)
    form_fields = grok.AutoFields(IFaculty)

class Content(FormWrapMixin, grok.Viewlet):
    """A viewlet that wraps the `AddFacultyForm`.
    """
    grok.viewletmanager(MainArea)
    grok.context(IFaculty)
    grok.view(Index)
    grok.require('waeup.manageUniversity')

    formview_name = 'displayform' # The name of the formview we want
                                  # to be rendered in this viewlet.


class ManageForm(grok.EditForm):
    """Manage the basic properties of a `Faculty` instance.
    """

    form_fields = grok.AutoFields(IFaculty)

    @property
    def label(self):
        # Set faculty name as form label
        return self.form_fields['title'].field.get(self.context)
    
    @grok.action('Save')
    def save(self, **data):
        self.applyData(self.context, **data)
        return
    
    @grok.action('Save and return')
    def saveAndReturn(self, **data):
        self.applyData(self.context, **data)
        self.redirect(self.url(self.context))
        return

    @grok.action('Cancel')
    def cancel(self, **data):
        self.redirect(self.url(self.context))
        return

class ManageFaculty(FormWrapMixin, grok.Viewlet):
    """A viewlet that renders the `ManageForm`.
    """
    grok.viewletmanager(MainArea)
    grok.context(IFaculty)
    grok.view(Manage)
    grok.require('waeup.manageUniversity')
    
    formview_name = 'manageform' # The name of the formview we want to
                                 # be rendered in this viewlet.

class ManageLink(grok.Viewlet):
    grok.viewletmanager(LeftSidebar)
    grok.context(IFaculty)
    grok.view(Index)
    grok.order(5)
    # This link is only displayed, if the user is allowed to use it!
    grok.require('waeup.manageUniversity')
    
    def render(self):
        return u'<div class="portlet"><a href="manage">Settings</a></div>'
